@@ -1,25 +1,30 @@
-import json
-import collections
 import argparse
-import tempfile
-import sys
+import contextlib
+import collections
+import enum
+import json
 import logging
 import os
-import shutil
 import re
-import enum
+import shutil
+import shlex
 import string
+import sys
+import tempfile
 import types
+from pathlib import Path
+from email.parser import BytesHeaderParser as EmailBytesHeaderParser
+from email.message import EmailMessage
 from typing import Tuple, List, Any, Dict
 from ruamel.yaml import YAML
 
-from vnm.consts import PROG, NAME, VAR_PREFIX, DESC
-
 log = logging.getLogger(__name__)
 if sys.version_info < (3, 6):
     print(f'CRITICAL: `{PROG}` MUST be run with Python 3.6 or higher. You are running v'+ ('.'.join(sys.version)))
     sys.exit(1)
 
+from vnm.consts import PROG, NAME, VAR_PREFIX, DESC
+
 import urllib.request
 import urllib.parse
 from typing import Optional, List
@@ -29,9 +34,11 @@ from wheel.wheelfile import WheelFile
 from vnm.repos import *
 from vnm.constraint import Constraint
 from vnm.package import Package
+from vnm.state import ConfigState
 from vnm.venv import activate, deactivate, setEnv, defined, unsetEnv
 
 VENVDIR = os.path.abspath('.venv')
+CORE_PACKAGES = ['pip', 'setuptools', 'wheel']
 
 yaml = YAML(typ='rt')
 
@@ -50,202 +57,60 @@ def getPython():
     subdir = 'Scripts' if os.name == 'nt' else 'bin'
     return os.path.abspath(os.path.join(VENVDIR, subdir, pyexe))
 
-def update_pip(args):
+def getPyPiVersionOf(packageid:str) -> str:
     xml = ''
-    with urllib.request.urlopen('https://pypi.org/rss/project/pip/releases.xml') as u:
+    with urllib.request.urlopen(f'https://pypi.org/rss/project/{packageid}/releases.xml') as u:
         #<title>20.2b1</title>
         xml = u.read().decode('utf-8')
     #print(xml)
     m = re.search(r'<title>([0-9\.]+)</title>', xml)
     assert m is not None
-    upstream_version = m[1]
+    return m[1]
 
-    o = check_output([getPython(), '-m', 'pip', '--version']).decode('utf-8')
-    m = re.search(r'pip ([^ ]+) from (.*)', o)
-    assert m is not None
-    pip_version = m[1]
+def update_package_manually(packageid, pkg: Optional[Package] = None, verbose: bool = False):
+    if pkg is None:
+        pkg = Package(packageid)
+    if pkg.id == '':
+        pkg.id = packageid
+    cstr = pkg.constraintsToStr()
+    if cstr == '':
+        cstr = '(Newest)'
+    print(f'Required {packageid} version: {cstr}')
+    print(f'  Updating {packageid}...')
+    #print(pkg.id, repr(pkg.serialize()))
+    cmd = [getPython(), '-m', 'pip', 'install', '-U']+pkg.toPipArgs()
+    if verbose: print('$', shlex.join(cmd))
+    check_call(cmd)
+    return pkg
 
-    print('Installed pip version: '+pip_version)
-    print('Newest pip version...: '+upstream_version)
-    if upstream_version!=pip_version:
-        print('Updating pip...')
-        check_call([getPython(), '-m', 'pip', 'install', '-U', 'pip'])
+def update_pip(pkg: Optional[Package] = None):
+    if pkg is not None and len(pkg.constraints) > 0:
+        update_package_manually('pip', pkg)
+    else:
+        upstream_version = getPyPiVersionOf('pip')
+        o = check_output([getPython(), '-m', 'pip', '--version']).decode('utf-8')
+        m = re.search(r'pip ([^ ]+) from (.*)', o)
+        assert m is not None
+        pip_version = m[1]
+        print(f'Installed pip version: {pip_version}')
+        print(f'Newest pip version: {upstream_version}')
+        if upstream_version!=pip_version:
+            print('  Updating pip...')
+            check_call([getPython(), '-m', 'pip', 'install', '-U', 'pip'])
+
+def check_core_pkgs(args: argparse.Namespace, cfg: ConfigState) -> None:
+    print('Checking core packages...')
+    update_pip(cfg.packages.get('pip'))
+    for pkgid in CORE_PACKAGES:
+        if pkgid == 'pip':
+            continue
+        update_package_manually(pkgid, cfg.packages.get(pkgid))
 
 class EPipOperation(enum.IntEnum):
     INSTALL = enum.auto()
     REINSTALL = enum.auto()
     UPGRADE = enum.auto()
 
-class ConfigState(object):
-    PIPJSON_VERSION = 1
-    VNMYML_VERSION = 3
-    STATE_FILENAME = 'vnm.state'
-    STATE_VERSION = '202007131859'
-    def __init__(self, args: Optional[argparse.Namespace] = None):
-        self.as_json: bool = False
-        self.dev_mode: bool = False
-        self.packages: Dict[str, Package] = {}
-        self.dev_packages: Dict[str, Package] = {}
-
-        self.yaml_upgrades: Dict[int, Callable] = {}
-        self.state_upgrades: Dict[Any, Callable] = {}
-
-        def addVnmYamlUpgrade(forVersion: int):
-            def _wrap(process: callable) -> callable:
-                self.yaml_upgrades[forVersion] = process
-                return process
-            return _wrap
-
-        @addVnmYamlUpgrade(1)
-        def vnmYmlv2(docs):
-            docs[0]={'VERSION': 2}
-            return docs
-
-        @addVnmYamlUpgrade(2)
-        def vnmYmlv3(docs):
-            newdoc = {
-                'version': 3,
-                'packages': docs[1]['packages']
-            }
-            if 'dev-packages' in docs[1]:
-                newdoc['dev-packages'] = docs[1]['dev-packages']
-            return [newdoc]
-
-        def addVnmStateUpgrade(forVersion: Any):
-            def _wrap(process: callable) -> callable:
-                self.state_upgrades[forVersion] = process
-                return callable
-            return _wrap
-
-        @addVnmStateUpgrade('202006122014')
-        def vnmState_2020_07_13_1859(docs):
-            return ['202007131859', docs[2]]
-
-    def getYamlVersion(self, docs) -> int:
-        # 202006122014
-        #if isinstance(docs, )
-        if isinstance(docs, list):
-            if isinstance(docs[0], int):
-                return docs[0]
-            elif isinstance(docs[0], dict):
-                if 'VERSION' in docs[0]:
-                    return docs[0]['VERSION']
-                if 'version' in docs[0]:
-                    return docs[0]['version']
-                return self.PIPJSON_VERSION
-            else:
-                raise ArgumentError('docs', 'Invalid type {0} for vnm.yml:docs[0]'.format(type(docs[0])))
-        elif isinstance(docs, dict):
-            return docs.get('version', self.VNMYML_VERSION)
-
-    def getStateVersion(self, docs) -> str:
-        if isinstance(docs, list):
-            if isinstance(docs[0], int):
-                return str(docs[0])
-            elif isinstance(docs[0], str):
-                return docs[0]
-            raise ArgumentError('docs', 'Invalid type {0} for vnm.state:docs[0]'.format(type(docs[0])))
-        else:
-            raise ArgumentError('docs', 'Invalid type {0} for vnm.state:docs'.format(type(docs)))
-
-    def load(self) -> None:
-        self.as_json = False
-        data = {}
-        statedata = {}
-        if os.path.isfile('vnm.yml'):
-            with open('vnm.yml', 'r') as f:
-                docs = yaml.load_all(f)
-                if isinstance(docs, types.GeneratorType):
-                    docs = list(docs)
-                version = self.getYamlVersion(docs)
-                while version in self.yaml_upgrades:
-                    print(f'WARNING: Updating vnm.yml to version {version+1}...')
-                    docs = self.yaml_upgrades[version](docs)
-                    version = self.getYamlVersion(docs)
-                #print(repr(docs))
-                data = docs[0]
-        elif os.path.isfile('pip.json'):
-            print('WARNING: pip.json is deprecated.')
-            self.as_json=True
-            with open('pip.json', 'r') as f:
-                data = json.load(f)
-
-        if os.path.isfile('vnm.state'):
-            with open('vnm.state', 'r') as f:
-                docs = list(yaml.load_all(f))
-                version = self.getStateVersion(docs)
-                while version in self.state_upgrades:
-                    docs = self.state_upgrades[version](docs)
-                    version = self.getStateVersion(docs)
-                    print(f'WARNING: Updated vnm.state to version {version}...')
-                statedata = docs[1]
-                self.dev_mode = statedata.get('dev-mode', False)
-
-        pkgdata = data.get('packages', {})
-        for k, v in pkgdata.items():
-            pkg = Package()
-            pkg.id = k
-            pkg.deserialize(v)
-            self.packages[k] = pkg
-
-        pkgdata = data.get('dev-packages', {})
-        for k, v in pkgdata.items():
-            pkg = Package()
-            pkg.id = k
-            pkg.deserialize(v)
-            self.dev_packages[k] = pkg
-
-    def get_packages(self) -> Dict[str, Package]:
-        o = self.packages.copy()
-        if self.dev_mode:
-            o.update(self.dev_packages)
-        return o
-
-    def get_all_packages(self) -> Dict[str, Package]:
-        return {**self.packages, **self.dev_packages}
-
-    def _serialize_pkg_dict(self, d):
-        o = collections.OrderedDict()
-        for k in sorted(d.keys()):
-            o[k]=d[k].serialize()
-        return dict(o)
-    def saveVNM(self, dry_run: bool=False) -> None:
-        if self.as_json:
-            data = {
-                'version': self.PIPJSON_VERSION,
-            }
-            if len(self.packages) > 0:
-                data['packages'] = self._serialize_pkg_dict(self.packages)
-            if len(self.dev_packages) > 0:
-                data['dev-packages'] = self._serialize_pkg_dict(self.dev_packages)
-            with open('pip.new.json' if dry_run else 'pip.json', 'w') as f:
-                json.dump(data, f, indent=2)
-        else:
-            data = {'version': self.VNMYML_VERSION}
-            if len(self.packages) > 0:
-                data['packages'] = self._serialize_pkg_dict(self.packages)
-            if len(self.dev_packages) > 0:
-                data['dev-packages'] = self._serialize_pkg_dict(self.dev_packages)
-            with open('vnm.new.yml' if dry_run else 'vnm.yml', 'w') as f:
-                f.write('# Generated by vnm.\n')
-                yaml.dump(data, f)
-
-    def saveState(self, dry_run: bool=False) -> None:
-        data={}
-        if self.dev_mode:
-            data['dev-mode']=True
-        # Mostly for debugging purposes.
-        if self.as_json:
-            data['as-json']=True
-        with open(self.STATE_FILENAME + ('.new' if dry_run else ''), 'w') as f:
-            f.write(f'# vnm state file version {self.STATE_VERSION}\n')
-            f.write('# @GENERATED automatically by vnm.\n')
-            f.write('# Do NOT edit this file, nor commit it to your VCS.\n')
-            f.write('###################################################\n')
-            f.write(f'{self.STATE_VERSION!r}\n')
-            f.write(f'---\n')
-            yaml.dump(data, f)
-
 def install_venv(args: argparse.Namespace, cfg: ConfigState, operation=EPipOperation.INSTALL) -> List[Package]:
     pipargs = []
     verb = 'Installing'
@@ -258,6 +123,9 @@ def install_venv(args: argparse.Namespace, cfg: ConfigState, operation=EPipOpera
 
     if len(cfg.packages) == 0 and len(cfg.dev_packages) == 0:
         return
+
+    #check_core_pkgs(args, cfg)
+
     print('Writing temporary requirements file...')
     workdir = os.getcwd()
     tempfilename = ''
@@ -267,7 +135,12 @@ def install_venv(args: argparse.Namespace, cfg: ConfigState, operation=EPipOpera
             os.remove('BROKEN-requirements.txt')
         with tempfile.NamedTemporaryFile(delete=False, mode='w') as f:
             tempfilename = f.name
-            for pid, pkg in cfg.get_packages().items():
+            packages = cfg.get_packages()
+            for pkgid in CORE_PACKAGES:
+                if pkgid in packages.keys():
+                    packages[pkgid].priority = -1
+            packages = packages.values()
+            for pkg in sorted(packages, key=lambda p: (p.priority, p.id)):
                 f.write(pkg.toRequirement()+'\n')
         print(f'{verb} packages from {tempfilename}...')
         check_call([getPython(), '-m', 'pip', 'install']+pipargs+['-r', tempfilename])
@@ -290,43 +163,80 @@ def install_venv(args: argparse.Namespace, cfg: ConfigState, operation=EPipOpera
     return all_packages
 
 def update_requirements(cfg, args, all_packages):
-    p = Popen([getPython(), '-m', 'pip', 'freeze'], stdout=PIPE, stderr=PIPE)
-    stdout, stderr = p.communicate()
-    suffix = '-dev' if cfg.dev_mode else ''
-    if p.returncode == 0:
-        all_package_names = []
-        pkgs = {}
-        for pkg in cfg.get_all_packages().values():
-            all_package_names += [pkg.id.lower()]
-            pkgs[pkg.id.lower()] = pkg
-        for line in stdout.splitlines():
-            #print(line)
-            #oline = line
-            line = line.strip()
-            p = Package(line.decode('utf-8'))
-            pkg = None
-            if p.id.lower() in all_package_names:
-                pkg = pkgs[p.id.lower()]
-            #else:
-            #    f.write(f'#{p.toRequirement()}\n')
-            if pkg is not None:
-                if len(pkg.constraints) == 0:
-                    pkg.constraints = p.constraints
-                if isinstance(pkg.repo, GitRepo) and isinstance(p.repo, GitRepo) and pkg.repo.refid != p.repo.refid:
-                    pkg.repo = p.repo
-            #print(line, pkg.id if pkg is not None else None, json.dumps(pkg.serialize()) if pkg is not None else None)
+    #print('update_requirements', cfg.generate_setup, cfg.generate_requirements)
+    if cfg.generate_requirements or cfg.generate_setup:
+        p = Popen([getPython(), '-m', 'pip', 'freeze'], stdout=PIPE, stderr=PIPE)
+        stdout, stderr = p.communicate()
+        suffix = '-dev' if cfg.dev_mode else ''
+        if p.returncode == 0:
+            all_package_names = []
+            pkgs = {}
+            for pkg in cfg.get_all_packages().values():
+                all_package_names += [pkg.id.lower()]
+                pkgs[pkg.id.lower()] = pkg
+            for line in stdout.splitlines():
+                #print(line)
+                #oline = line
+                line = line.strip()
+                p = Package(line.decode('utf-8'))
+                pkg = None
+                if p.id.lower() in CORE_PACKAGES:
+                    continue
+                if p.id.lower() in all_package_names:
+                    pkg = pkgs[p.id.lower()]
+                #else:
+                #    f.write(f'#{p.toRequirement()}\n')
+                if pkg is not None:
+                    if len(pkg.constraints) == 0:
+                        pkg.constraints = p.constraints
+                    if isinstance(pkg.repo, GitRepo) and isinstance(p.repo, GitRepo) and pkg.repo.refid != p.repo.refid:
+                        pkg.repo = p.repo
+                #print(line, pkg.id if pkg is not None else None, json.dumps(pkg.serialize()) if pkg is not None else None)
+
+            for pkgid in CORE_PACKAGES:
+                p = Package(pkgid)
+
+                c = Constraint()
+                c.setEquality('==')
+                c.setVersion(get_local_pkg_version(p.id))
+                #print(f'Adding constraint {c.toRequirement()} to package {p.id}')
+                p.constraints += [c]
+
+                pkg = None
+                if pkgid in all_package_names:
+                    pkg = pkgs[p.id]
+
+                if pkg is not None:
+                    if len(pkg.constraints) == 0:
+                        pkg.constraints = p.constraints
+                    if isinstance(pkg.repo, GitRepo) and isinstance(p.repo, GitRepo) and pkg.repo.refid != p.repo.refid:
+                        pkg.repo = p.repo
+
+            all_release_pkgs = [k.lower() for k in cfg.packages.keys()]
+            all_pkgs = all_release_pkgs+[k.lower() for k in cfg.dev_packages.keys()]
+
+            if cfg.generate_setup:
+                print('Writing setup.cfg')
+                cfg.update_setup(args, all_release_pkgs, all_pkgs, pkgs)
+
+            if cfg.generate_requirements:
+                print('Writing requirements')
+                fdev = contextlib.nullcontext()
+                if cfg.dev_mode:
+                    fdev = open(f'requirements-dev.new.txt' if args.dry_run else f'requirements-dev.txt', 'w')
+                with open(f'requirements.new.txt' if args.dry_run else f'requirements.txt', 'w') as frel:
+                    with fdev:
+                        frel.write(f'# Generated by vnm\n')
+                        if cfg.dev_mode:
+                            fdev.write(f'# Generated by vnm\n')
+                        for pkg in sorted(pkgs.values(), key=lambda p: p.priority):
+                            if pkg.id.lower() in all_release_pkgs:
+                                #frel.write(f'#{pkg.priority}\n')
+                                frel.write(f'{pkg.toRequirement()}\n')
+                            if cfg.dev_mode and pkg.id.lower() in all_pkgs:
+                                fdev.write(f'{pkg.toRequirement()}\n')
+
 
-        all_release_pkgs = [k.lower() for k in cfg.packages.keys()]
-        all_pkgs = all_release_pkgs+[k.lower() for k in cfg.dev_packages.keys()]
-        with open(f'requirements.new.txt' if args.dry_run else f'requirements.txt', 'w') as f:
-            for pkg in pkgs.values():
-                if pkg.id.lower() in all_release_pkgs:
-                    f.write(f'{pkg.toRequirement()}\n')
-        if cfg.dev_mode:
-            with open(f'requirements-dev.new.txt' if args.dry_run else f'requirements-dev.txt', 'w') as f:
-                for pkg in pkgs.values():
-                    if pkg.id.lower() in all_pkgs:
-                        f.write(f'{pkg.toRequirement()}\n')
 def main():
     logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')
     argp = argparse.ArgumentParser(prog=PROG, description=DESC)
@@ -339,12 +249,18 @@ def main():
     p_upgrade_config = subp.add_parser('upgrade-config', help='Upgrade from pip.json to vnm.yml.')
     p_upgrade_config.set_defaults(cmd=_cmd_upgrade_config)
 
+    p_import_setup = subp.add_parser('import-setup', help='Import setup.cfg')
+    p_import_setup.add_argument('filename', default='setup.cfg', nargs='?', help="Alternate name for the .cfg file.")
+    p_import_setup.set_defaults(cmd=_cmd_import_setup)
+
     p_install = subp.add_parser('install', aliases=['i'], help='Install all packages defined in vnm.yml')
     p_install.add_argument('--dev', '-d', action='store_true', default=False, help='Install in development mode. (Adds dev-packages.)')
     p_install.set_defaults(cmd=_cmd_install)
 
     p_init = subp.add_parser('init', help='Set up an initial .venv and vnm.yml.')
     p_init.add_argument('--clobber', action='store_true', default=False, help="Forcefully reinstall .venv and vnm.yml.")
+    p_init.add_argument('--git', action='store_true', default=False, help="Setup git and add necessary excludes and ignores.")
+    p_init.add_argument('--lfs', action='store_true', default=False, help="Setup git lfs, as well.")
     p_init.set_defaults(cmd=_cmd_init)
 
     p_add = subp.add_parser('add', help='Add a dependency to vnm.yml')
@@ -417,12 +333,19 @@ def _cmd_add(args) -> None:
 
     # Same as _cmd_install().
     activate(VENVDIR)
-    update_pip(args)
+    check_core_pkgs(args, cfg)
     all_packages = install_venv(args, cfg)
     deactivate(VENVDIR)
 
     update_requirements(cfg, args, all_packages)
 
+def get_local_pkg_version(pkgid: str) -> str:
+    # I don't know why they did this.
+    o: bytes = check_output([getPython(), '-m', 'pip', 'show', pkgid])
+    headers: EmailMessage = EmailBytesHeaderParser().parsebytes(o)
+    #print(repr(dict(headers)))
+    return headers.get('Version')
+
 def _cmd_freeze(args) -> None:
     require_venv()
 
@@ -436,8 +359,7 @@ def _cmd_freeze(args) -> None:
             print(f'WARNING: Package {pkg.id!r} not specified in vnm.yml/pip.json.')
         pkgs_to_freeze += [pkg.id]
 
-
-    p = Popen([getPython(), '-m', 'pip', 'freeze'], stdout=PIPE, stderr=PIPE)
+    p = Popen([getPython(), '-m', 'pip', 'freeze', '--all'], stdout=PIPE, stderr=PIPE)
     stdout, stderr = p.communicate()
     suffix = '-dev' if cfg.dev_mode else ''
     if p.returncode == 0:
@@ -461,15 +383,35 @@ def _cmd_freeze(args) -> None:
 
             if len(pkgs_to_freeze) == 0 or pkg.id in pkgs_to_freeze:
                 if len(p.constraints) == 0:
-                    print(f'WARNING: Package {p.id} has no version info')
-                    continue
+                    if p.repo and hasattr(p.repo, 'refid') and p.repo.reftype == EGitRefType.COMMIT:
+                        c = Constraint()
+                        c.setEquality('==')
+                        c.setVersion(p.repo.refid)
+                        print(f'Adding constraint {c.toRequirement()}')
+                        p.constraints += [c]
+                        print(f'Removing refid {p.repo.refid}')
+                        p.repo.refid = None
+                        p.repo.reftype = None
+                    else:
+                        print(f'WARNING: Package {p.id} has no version info. Grabbing from pip show instead.')
+                        c = Constraint()
+                        c.setEquality('==')
+                        c.setVersion(get_local_pkg_version(p.id))
+                        print(f'Adding constraint {c.toRequirement()}')
+                        p.constraints += [c]
+                        #continue
                 if len(p.constraints) > 1:
                     pc = ', '.join([x.toRequirement() for x in p.constraints])
-                    print(f'WARNING: Package {p.id} has a bunch of constraints. ({pc})')
-                    continue
+                    print(f'WARNING: Package {p.id} has a bunch of constraints. ({pc}) Grabbing from pip show instead.')
+                    c = Constraint()
+                    c.setEquality('==')
+                    c.setVersion(get_local_pkg_version(p.id))
+                    print(f'Settng constraints to {c.toRequirement()}')
+                    p.constraints = [c]
+                    #continue
                 pkg.freezeAt(p.constraints[0])
-                if isinstance(pkg.repo, GitRepo) and isinstance(p.repo, GitRepo) and pkg.repo.refid != p.repo.refid:
-                    pkg.repo = p.repo
+                if isinstance(pkg.repo, GitRepo) and isinstance(p.repo, GitRepo):
+                    pkg.repo.uri = p.repo.uri
 
                 if pkg.id in cfg.dev_packages.keys():
                     print(f'Freezing dev package {pkg.id!r}')
@@ -484,7 +426,7 @@ def _cmd_freeze(args) -> None:
 
     # Same as _cmd_install().
     activate(VENVDIR)
-    update_pip(args)
+    check_core_pkgs(args, cfg)
     all_packages = install_venv(args, cfg)
     deactivate(VENVDIR)
 
@@ -521,7 +463,28 @@ def _cmd_thaw(args) -> None:
 
     # Same as _cmd_install().
     activate(VENVDIR)
-    update_pip(args)
+    check_core_pkgs(args, cfg)
+    all_packages = install_venv(args, cfg)
+    deactivate(VENVDIR)
+
+    update_requirements(cfg, args, all_packages)
+
+def _cmd_import_setup(args) -> None:
+    require_venv()
+
+    cfg = ConfigState()
+    cfg.load()
+
+    path = Path(args.filename)
+    print('Importing {path}...')
+    cfg.import_setup(path)
+
+    cfg.saveState()
+    cfg.saveVNM()
+
+    # Same as _cmd_install().
+    activate(VENVDIR)
+    check_core_pkgs(args, cfg)
     all_packages = install_venv(args, cfg)
     deactivate(VENVDIR)
 
@@ -551,7 +514,7 @@ def _cmd_remove(args) -> None:
 
     # Same as _cmd_install().
     activate(VENVDIR)
-    update_pip(args)
+    check_core_pkgs(args, cfg)
     call([getPython(), '-m', 'pip', 'uninstall', '-y']+rmpkgs)
     deactivate(VENVDIR)
 
@@ -565,7 +528,7 @@ def _cmd_upgrade(args):
     cfg.load()
 
     activate(VENVDIR)
-    update_pip(args)
+    check_core_pkgs(args, cfg)
     all_packages = install_venv(args, cfg, operation=EPipOperation.UPGRADE)
     deactivate(VENVDIR)
 
@@ -617,9 +580,9 @@ def _cmd_install(args):
         if os.path.isdir('.venv'):
             clear_cmd += ['--clear']
         clear_cmd += ['.venv']
-        clear_cmd = ' '.join(clear_cmd)
-        print(f'Running {clear_cmd}...')
-        check_call(clear_cmd)
+        clear_cmd_str = ' '.join(clear_cmd)
+        print(f'Running {clear_cmd_str}...')
+        check_call(clear_cmd, shell=False)
     #require_venv()
     require_pipjson()
 
@@ -629,7 +592,7 @@ def _cmd_install(args):
     cfg.saveState()
 
     activate(VENVDIR)
-    update_pip(args)
+    check_core_pkgs(args, cfg)
     all_packages = install_venv(args, cfg)
     deactivate(VENVDIR)
 
@@ -657,7 +620,19 @@ def _cmd_init(args):
     cfg.saveVNM()
     cfg.saveState()
 
-    #update_pip(args)
+    if args.git:
+        if not os.path.isdir('.git'):
+            print('Running git init...')
+            check_call(['git', 'init'])
+            if args.lfs:
+                print('Running git lfs install...')
+                check_call(['git', 'lfs', 'install'])
+        if not os.path.isfile('.gitignore'):
+            print('Writing .gitignore...')
+            with open('.gitignore', 'w') as f:
+                f.write('# vnm files\n/vnm.state\n/.venv/\n\n# general python stuff\n*.py[ioc]\n/*.egg-info/\n')
+
+    #check_core_pkgs(args, cfg)
     print( '********************************************************************')
     print(f'.venv for {NAME} installed!')
     print( 'Next steps:')
@@ -716,6 +691,7 @@ def _cmd_activate_nt(args):
     hide_script(os.path.join(VENVDIR, 'Scripts', 'deactivate.bat'))
     code = -1
     try:
+        #code = call(['cmd.exe', '/k', os.path.join(VENVDIR, 'Scripts', 'deactivate.bat')], shell=True)
         code = call(['cmd.exe'], shell=True)
     finally:
         print(f'{PROG}: cmd.exe exited with code {code}')
