@@ -1,7 +1,7 @@
 from typing import List
 class Repo(object):
     TYPEID = ''
-    def __init__(self):
+    def __init__(self) -> None:
         pass
 
     def serialize(self) -> dict:
@@ -18,3 +18,9 @@ class Repo(object):
 
     def shouldDisplayConstraints(self) -> bool:
         return True
+
+    def freeze(self, at: 'Constraint') -> None:
+        return
+
+    def thaw(self) -> None:
+        return
